﻿using ForumContracts.BindingModels;
using ForumContracts.SearchModels;
using ForumContracts.StoragesContracts;
using ForumContracts.ViewModels;
using ForumDatabase.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ForumDatabase.Implements
{
    public class CategoryStorage : ICategoryStorage
    {
        public CategoryViewModel? Delete(CategoryBindingModel model)
        {
            using var context = new ForumDatabase();
            var element = context.Categories.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Categories.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        public CategoryViewModel? GetElement(CategorySearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new ForumDatabase();
            return context.Categories
                .FirstOrDefault(x =>
                (!string.IsNullOrEmpty(model.Name) && x.Name ==
                model.Name) ||
                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public List<CategoryViewModel> GetFilteredList(CategorySearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            using var context = new ForumDatabase();
            return context.Categories
                .Where(x => x.Name.Contains(model.Name))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<CategoryViewModel> GetFullList()
        {
            using var context = new ForumDatabase();
            return context.Categories
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public CategoryViewModel? Insert(CategoryBindingModel model)
        {
            var newCategory = Category.Create(model);
            if (newCategory == null)
            {
                return null;
            }
            using var context = new ForumDatabase();
            context.Categories.Add(newCategory);
            context.SaveChanges();
            return newCategory.GetViewModel;
        }

        public CategoryViewModel? Update(CategoryBindingModel model)
        {
            using var context = new ForumDatabase();
            var component = context.Categories.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            context.SaveChanges();
            return component.GetViewModel;
        }
    }
}
