﻿using ForumContracts.BindingModels;
using ForumContracts.SearchModels;
using ForumContracts.StoragesContracts;
using ForumContracts.ViewModels;
using ForumDatabase.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.ConstrainedExecution;
using System.Text;
using System.Threading.Tasks;

namespace ForumDatabase.Implements
{
    public class UserStorage : IUserStorage
    {
        public UserViewModel? Delete(UserBindingModel model)
        {
            using var context = new ForumDatabase();
            var element = context.Users.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Users.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        public UserViewModel? GetElement(UserSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Username) && !model.Id.HasValue && string.IsNullOrEmpty(model.Email))
            {
                return null;
            }
            using var context = new ForumDatabase();
            return context.Users
                .Include(x => x.Role)
                .FirstOrDefault(x =>
                (!string.IsNullOrEmpty(model.Username) && x.Username ==
                model.Username) ||
                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public List<UserViewModel> GetFilteredList(UserSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Username) || string.IsNullOrEmpty(model.Email))
            {
                return new();
            }
            using var context = new ForumDatabase();
            return context.Users
                .Include(x => x.Role)
                .Where(x => x.Username.Contains(model.Username))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<UserViewModel> GetFullList()
        {
            using var context = new ForumDatabase();
            return context.Users
                .Include(x => x.Role)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public UserViewModel? Insert(UserBindingModel model)
        {
            var newUser = User.Create(model);
            if (newUser == null)
            {
                return null;
            }
            using var context = new ForumDatabase();
            context.Users.Add(newUser);
            context.SaveChanges();
            return newUser.GetViewModel;
        }

        public UserViewModel? Update(UserBindingModel model)
        {
            using var context = new ForumDatabase();
            var component = context.Users.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            context.SaveChanges();
            return component.GetViewModel;
        }

        public void UserInsertList(int num, List<RoleViewModel> roles)
        {
            Random rnd = new Random();
            using var context = new ForumDatabase();
            for (int i = 0; i < num; ++i)
            {
                var model = new UserBindingModel
                {
                    Id = 0,
                    Username = rnd.Next(1000).ToString(),
                    Email = rnd.Next(1000).ToString(),
                    Password = rnd.Next(1000).ToString(),
                    RoleId = roles[rnd.Next(roles.Count)].Id,
                };
                context.Users.Add(User.Create(model));
            }
            context.SaveChanges();
        }
    }
}
